/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/**************************************************************************//**
* @file         GWP0700CNWV04.c
* @version      0
* $Rev:
* $Date::
* @brief        LCD panel for vdc5 channel 0 function
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/



#include   "GWP0700CNWV04.h"
#ifdef GWP0700CNWV04
#include    "r_typedefs.h"
/* Device driver header */
#include "dev_drv.h"


#include    "iodefine.h"
#include 	"gpio_iobitmask.h"
#include    "rza_io_regrw.h"
#include    "riic_rskrza1h.h"

typedef struct
{
	uint16_t	cur_x;
	uint16_t	cur_y;
	uint16_t	cur_id;
}S_TP;

#define NUM_TP	5
#define HYSTVALUE 5

S_TP TP_Table[NUM_TP];



/******************************************************************************
Macro definitions
******************************************************************************/
/* Port 11 */
#define LCD_PORT11_5TH    (0x90FFu)
#define LCD_PORT11_OUTPUT (0x4C00u)
/* Port 10 */
#define LCD_PORT10_5TH (0xFFFFu)

#define STP91_BIT1  (0x02u)

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
static void SetLcdPanel_Ch0(const uint32_t user_num);


/**************************************************************************//**
 * @brief
 * @param[in]   void
 * @retval      None
******************************************************************************/
void SetLcdPanel_Ch0(const uint32_t user_num)
{
    volatile uint32_t   dummy_read;
    uint32_t            reg_data;

   /*                   : LCD0_CLK              ... P11_15, 5th alternative function
            DE          : LCD0_TCON2            ... P11_12, 5th alternative function
        LCD0_DATA
            R[5:0]      : LCD0_DATA[17:12]      ... P10_6 ~ P10_11, 5th alternative function
            G[5:0]      : LCD0_DATA[11:8]       ... P10_12 ~ P10_15, 5th alternative function
                        : LCD0_DATA[7:6]        ... P11_0 ~ P11_1, 5th alternative function
            B[5:0]      : LCD0_DATA[5:0]        ... P11_2 ~ P11_7, 5th alternative function
    */
    /* Port 11 */
    reg_data        = (uint32_t)GPIO.PMC11 & (uint32_t)~LCD_PORT11_5TH;
    GPIO.PMC11      = (uint16_t)reg_data;
    reg_data        = (uint32_t)GPIO.PMC11;
    dummy_read      = reg_data;
    (void)dummy_read; /* satisfy the compiler */
    /* PFCAE11, PFCE11, PFC11 ... 5th alternative function
       PIPC11, PMC11
       b15      : P11_15
       b12      : P11_12
       b7:b0    : P11_7 ~ P11_0 */
    reg_data        = (uint32_t)GPIO.PFCAE11 | (uint32_t)LCD_PORT11_5TH;
    GPIO.PFCAE11    = (uint16_t)reg_data;
    reg_data        = (uint32_t)GPIO.PFCE11  & (uint32_t)~LCD_PORT11_5TH;
    GPIO.PFCE11     = (uint16_t)reg_data;
    reg_data        = (uint32_t)GPIO.PFC11   & (uint32_t)~LCD_PORT11_5TH;
    GPIO.PFC11      = (uint16_t)reg_data;
    reg_data        = (uint32_t)GPIO.PIPC11  | (uint32_t)LCD_PORT11_5TH;
    GPIO.PIPC11     = (uint16_t)reg_data;
    reg_data        = (uint32_t)GPIO.PMC11   | (uint32_t)LCD_PORT11_5TH;
    GPIO.PMC11      = (uint16_t)reg_data;

    /* MODE		= P11_14 // MODE  = 1
       VSYNC	= P11_10 // VSYNC = 1
       HSYNC    = P11_11 // HSYNC = 1
     */
    reg_data        = (uint32_t)GPIO.PMC11 & (uint32_t)~LCD_PORT11_OUTPUT;
    GPIO.PMC11      = (uint16_t)reg_data;
    reg_data        = (uint32_t)GPIO.PMC11;
    dummy_read      = reg_data;
    (void)dummy_read; /* satisfy the compiler */

    reg_data        = (uint32_t)GPIO.P11 | (uint32_t)LCD_PORT11_OUTPUT;
    GPIO.P11    = (uint16_t)reg_data;

    reg_data        = (uint32_t)GPIO.PM11 | (uint32_t)~LCD_PORT11_OUTPUT;
    GPIO.PM11    = (uint16_t)reg_data;


    /* Port 10 */
    reg_data        = (uint32_t)GPIO.PMC10   & (uint32_t)~LCD_PORT10_5TH;
    GPIO.PMC10      = (uint16_t)reg_data;
    reg_data        = (uint32_t)GPIO.PMC10;
    dummy_read      = reg_data;
    (void)dummy_read; /* satisfy the compiler */

    /* PFCAE10, PFCE10, PFC10 ... 5th alternative function
       PIPC10, PMC10
       b15:b6   : P10_15 ~ P10_6 */
    reg_data        = (uint32_t)GPIO.PFCAE10 | (uint32_t)LCD_PORT10_5TH;
    GPIO.PFCAE10    = (uint16_t)reg_data;
    reg_data        = (uint32_t)GPIO.PFCE10  & (uint32_t)~LCD_PORT10_5TH;
    GPIO.PFCE10     = (uint16_t)reg_data;
    reg_data        = (uint32_t)GPIO.PFC10   & (uint32_t)~LCD_PORT10_5TH;
    GPIO.PFC10      = (uint16_t)reg_data;
    reg_data        = (uint32_t)GPIO.PIPC10  | (uint32_t)LCD_PORT10_5TH;
    GPIO.PIPC10     = (uint16_t)reg_data;
    reg_data        = (uint32_t)GPIO.PMC10   | (uint32_t)LCD_PORT10_5TH;
    GPIO.PMC10      = (uint16_t)reg_data;

 }   /* End of function SetLcdPanel_Ch0() */



vdc5_error_t InitVDC5(void* FrameBufferAddress, uint16_t FrameBufferWidth, uint16_t FrameBufferHeight, vdc5_gr_format_t vdc5_gr_format)
{
	vdc5_error_t            error;
    vdc5_init_t             init;
    vdc5_sync_ctrl_t        sync_ctrl;
    vdc5_output_t           output;
    vdc5_lcd_tcon_timing_t  outctrl[VDC5_LCD_TCONSIG_NUM];
    vdc5_read_t             read;
    vdc5_gr_disp_sel_t      gr_disp_sel[VDC5_GR_TYPE_NUM];
    vdc5_start_t            start;
    unsigned int            i;
    volatile uint8_t        dummy_read;
    uint32_t                reg_data;
    vdc5_disp_calibration_t	vdc5_disp_calibration;
    vdc5_calibr_bright_t	vdc5_calibr_bright;
    vdc5_calibr_contrast_t  vdc5_calibr_contrast;
    vdc5_calibr_dither_t	vdc5_calibr_dither;

   /* Standby control register 9 (STBCR9)
      b1      ------0-;  MSTP91 : 0 : Video display controller channel 0 & LVDS enable */
      /* Standby control register 9 (STBCR9)
            b1      ------0-;  MSTP91 : 0 : Video display controller channel 0 & LVDS enable */
    reg_data    = ((uint32_t)CPG.STBCR9 & (uint32_t)~STP91_BIT1);
    CPG.STBCR9  = (uint8_t)reg_data;
    /* In order to reflect the change, a dummy read should be done. */
    dummy_read = CPG.STBCR9;
    (void)dummy_read; /* satisfy the compiler */

    init.panel_icksel   = VDC5_PANEL_ICKSEL_PERI; /* Panel clock select: P1Clk = 66,65 MHz */
    init.panel_dcdr     = VDC5_PANEL_CLKDIV_1_2;  /* Panel clock frequency division ratio: 33.325 MHz = 33,33 MHz / 2. Typical frequency of display is 33.3 MHz  */
    init.lvds           = NULL;                   /* LVDS parameter */
    error = R_VDC5_Initialize(VDC5_CHANNEL_0, &init, SetLcdPanel_Ch0, 0);
    if (error != VDC5_OK) return error;

    sync_ctrl.res_vs_sel    = VDC5_ON;               /* Internally generated free-running Vsync signal */
    sync_ctrl.res_vs_in_sel = VDC5_RES_VS_IN_SEL_SC0;
    sync_ctrl.res_fv        = LCD_tv - 1;                /* Free-running Vsync period setting */
    sync_ctrl.res_fh        = LCD_th - 1;                /* Hsync period of internal Hsync counter */
    sync_ctrl.res_vsdly     = 0u;                        /* Vsync signal delay control */
    sync_ctrl.res_f.vs      = LCD_tvb; 					// vertical display start position
    sync_ctrl.res_f.vw      = LCD_tvd;					// vertical display area
    sync_ctrl.res_f.hs      = LCD_thb;        			// horizontal display start position
    sync_ctrl.res_f.hw      = LCD_thd;					// horizontal display area
    sync_ctrl.vsync_cpmpe   = NULL;                 /* Vsync signal compensation */
    error = R_VDC5_SyncControl(VDC5_CHANNEL_0, &sync_ctrl);
    if (error != VDC5_OK) return error;

    output.tcon_half        = (LCD_tv -1) / 2;        /* TCON reference timing, 1/2fH timing */
    output.tcon_offset      = 0;                     /* TCON reference timing, offset Hsync signal timing */

    for (i = 0; i < VDC5_LCD_TCONSIG_NUM; i++)
      output.outctrl[i] = NULL;

    // VSync Enable
    outctrl[VDC5_LCD_TCONSIG_STVB_VE].tcon_hsvs =     LCD_tvb * 2;		// vertical pulse start position
    outctrl[VDC5_LCD_TCONSIG_STVB_VE].tcon_hwvw =     LCD_tvd * 2;       // vertical enable width
    outctrl[VDC5_LCD_TCONSIG_STVB_VE].tcon_inv =      VDC5_SIG_POL_NOT_INVERTED;
    outctrl[VDC5_LCD_TCONSIG_STVB_VE].tcon_pin =      VDC5_LCD_TCON_PIN_NON;
    outctrl[VDC5_LCD_TCONSIG_STVB_VE].outcnt_edge =   VDC5_EDGE_RISING;
    output.outctrl[VDC5_LCD_TCONSIG_STVB_VE] =        &outctrl[VDC5_LCD_TCONSIG_STVB_VE];

    // HSync Enable
    outctrl[VDC5_LCD_TCONSIG_STB_LP_HE].tcon_hsvs =   LCD_thb;					// horizontal pulse start position
    outctrl[VDC5_LCD_TCONSIG_STB_LP_HE].tcon_hwvw =   LCD_thd; 					// horizontal enable width
    outctrl[VDC5_LCD_TCONSIG_STB_LP_HE].tcon_hs_sel = VDC5_LCD_TCON_REFSEL_HSYNC;  // internal VDC5 free-running vsync counter is used
    outctrl[VDC5_LCD_TCONSIG_STB_LP_HE].tcon_inv =    VDC5_SIG_POL_NOT_INVERTED;
    outctrl[VDC5_LCD_TCONSIG_STB_LP_HE].tcon_pin =    VDC5_LCD_TCON_PIN_NON;
    outctrl[VDC5_LCD_TCONSIG_STB_LP_HE].outcnt_edge = VDC5_EDGE_RISING;
    output.outctrl[VDC5_LCD_TCONSIG_STB_LP_HE] =      &outctrl[VDC5_LCD_TCONSIG_STB_LP_HE];

    // DE
    outctrl[VDC5_LCD_TCONSIG_DE].tcon_inv =    VDC5_SIG_POL_NOT_INVERTED;  // DE is positive logic
    outctrl[VDC5_LCD_TCONSIG_DE].tcon_pin =    VDC5_LCD_TCON_PIN_2;
    outctrl[VDC5_LCD_TCONSIG_DE].outcnt_edge = VDC5_EDGE_RISING;
    output.outctrl[VDC5_LCD_TCONSIG_DE] =      &outctrl[VDC5_LCD_TCONSIG_DE];

    output.outcnt_lcd_edge  = VDC5_EDGE_RISING;             /* Output phase control of LCD_DATAxx pin */
    output.out_endian_on    = VDC5_OFF;                     /* Bit endian change on/off control */
    output.out_swap_on      = VDC5_OFF;                     /* B/R signal swap on/off control */
    output.out_format       = VDC5_LCD_OUTFORMAT_RGB888;    /* Output format select */
    output.bg_color         = (uint32_t)0uL;       /* Background color in 24-bit RGB color format */
    error = R_VDC5_DisplayOutput(VDC5_CHANNEL_0, &output);
    if (error != VDC5_OK) return error;

    vdc5_calibr_bright.pbrt_g = 512;
    vdc5_calibr_bright.pbrt_b = 512;
    vdc5_calibr_bright.pbrt_r = 512;

    vdc5_calibr_contrast.cont_g = 128;
    vdc5_calibr_contrast.cont_b = 128;
    vdc5_calibr_contrast.cont_r = 128;

    //vdc5_calibr_dither.pdth_sel = VDC5_PDTH_MD_TRU;  // Truncate
    //vdc5_calibr_dither.pdth_sel = VDC5_PDTH_MD_RDOF; // Round-off
    vdc5_calibr_dither.pdth_sel = VDC5_PDTH_MD_2X2;  // 2x2 pattern dither
	//vdc5_calibr_dither.pdth_sel = VDC5_PDTH_MD_RAND;  // Random pattern dither
    vdc5_calibr_dither.pdth_pa = 3; // Pattern value (A) of 2x2 pattern dither 0 to 3. Referenced only when pdth_sel is set to VDC5_PDTH_MD_2X2.
    vdc5_calibr_dither.pdth_pb = 0; // Pattern value (B) of 2x2 pattern dither 0 to 3. Referenced only when pdth_sel is set to VDC5_PDTH_MD_2X2.
    vdc5_calibr_dither.pdth_pc = 2; // Pattern value (C) of 2x2 pattern dither 0 to 3. Referenced only when pdth_sel is set to VDC5_PDTH_MD_2X2.
    vdc5_calibr_dither.pdth_pd = 1; // Pattern value (D) of 2x2 pattern dither 0 to 3. Referenced only when pdth_sel is set to VDC5_PDTH_MD_2X2.

    vdc5_disp_calibration.bright 		= &vdc5_calibr_bright;
    vdc5_disp_calibration.contrast		= &vdc5_calibr_contrast;
    vdc5_disp_calibration.panel_dither	= &vdc5_calibr_dither;
    vdc5_disp_calibration.route			= VDC5_CALIBR_ROUTE_BCG; // Brightness -> contrast -> gamma correction
    //vdc5_disp_calibration.route			= VDC5_CALIBR_ROUTE_GBC; // Gamma correction -> brightness -> contrast

    error = R_VDC5_DisplayCalibration(VDC5_CHANNEL_0, &vdc5_disp_calibration);
    if (error != VDC5_OK) return error;

    read.gr_ln_off_dir  = VDC5_GR_LN_OFF_DIR_INC;   /* Line offset address direction of the frame buffer */
    read.gr_flm_sel     = VDC5_GR_FLM_SEL_FLM_NUM;  /* Selects a frame buffer address setting signal */
    read.gr_bst_md      = VDC5_BST_MD_32BYTE;       /* Frame buffer burst transfer mode */
    read.gr_base        = (void*)FrameBufferAddress;  /* Frame buffer base address */
    read.width_read_fb  = NULL;                     /* Width of the image read from frame buffer */
    read.adj_sel        = VDC5_OFF;                 /* Measures to decrease the influence by folding pixels/lines (on/off) */
    read.gr_grc.vs      = LCD_tvb;		// vertical display start position
    read.gr_grc.vw      = FrameBufferHeight;		// typical frame buffer height = vertical display width
    read.gr_grc.hs      = LCD_thb;	    // horizontal display start position
    read.gr_grc.hw      = FrameBufferWidth;       // typical frame buffer width = horizontal display width

    read.gr_format = vdc5_gr_format;
    switch (vdc5_gr_format)
    {
    case VDC5_GR_FORMAT_RGB565:
		read.gr_ln_off = (unsigned long)FrameBufferWidth*2;     /* Frame buffer line offset address */
    	read.gr_rdswa  = VDC5_WR_RD_WRSWA_32_16BIT;
    	break;

    case VDC5_GR_FORMAT_RGB888:
    case VDC5_GR_FORMAT_ARGB8888:
		read.gr_ln_off = (unsigned long)FrameBufferWidth*4;     /* Frame buffer line offset address */
		read.gr_rdswa  = VDC5_WR_RD_WRSWA_32BIT;
		break;

    default:
    	return !VDC5_OK;
    }
	error = R_VDC5_ReadDataControl(VDC5_CHANNEL_0, VDC5_LAYER_ID_2_RD, &read);
    if (error != VDC5_OK) return error;

    gr_disp_sel[VDC5_GR_TYPE_GR0] = VDC5_DISPSEL_IGNORED;
    gr_disp_sel[VDC5_GR_TYPE_GR1] = VDC5_DISPSEL_IGNORED;
    gr_disp_sel[VDC5_GR_TYPE_GR2] = VDC5_DISPSEL_CURRENT;
    gr_disp_sel[VDC5_GR_TYPE_GR3] = VDC5_DISPSEL_IGNORED;
    gr_disp_sel[VDC5_GR_TYPE_VIN] = VDC5_DISPSEL_IGNORED;
    gr_disp_sel[VDC5_GR_TYPE_OIR] = VDC5_DISPSEL_IGNORED;
    start.gr_disp_sel = gr_disp_sel;
    error = R_VDC5_StartProcess(VDC5_CHANNEL_0, VDC5_LAYER_ID_ALL, &start);
    if (error != VDC5_OK) return error;

    return VDC5_OK;
}


void HorzLimit (int16_t *x)
{
  *x = (*x < 0) ? 0 : *x;
  *x = (*x >= LCD_thd) ? (int16_t)(LCD_thd - 1) : (int16_t) *x;
}

void VertLimit (int16_t *y)
{
  *y = (*y < 0) ? 0 : *y;
  *y = (*y >= LCD_tvd) ? (int16_t)(LCD_tvd - 1) : (int16_t) *y;
}


void TouchDisplayUpdate(void)
{
  uint8_t  data[0x20];
  uint8_t  index;
  int32_t  ret;

  if (g_touch_read_request)
  {
    /* no more touch points until next interrupt */
   	g_touch_read_request = 0;
    ret = R_RIIC_rza1h_rsk_read(DEVDRV_CH_0, RZA1H_APP_I2C_FT5216, 0, 32, data);
    if (DEVDRV_SUCCESS == ret)
    {
      for(index = 0; index < NUM_TP; index++)
      {
        if(data[2] > index)
        {
          ret += (1<<index);
          TP_Table[index].cur_x  = (((data[3 + (index * 6)] & 0x7) << 8) + data[4 + (index * 6)]);
          TP_Table[index].cur_y  = (((data[5 + (index * 6)] & 0x7) << 8) + data[6 + (index * 6)]);
          TP_Table[index].cur_id = (((data[5 + (index * 6)] & 0xF0) >> 4));
          TP_Table[index].cur_x  = (LCD_thd - TP_Table[index].cur_x);
          TP_Table[index].cur_y  = (LCD_tvd - TP_Table[index].cur_y);
        }
				else
				{
					TP_Table[index].cur_x  = 0;
					TP_Table[index].cur_y  = 0;
          TP_Table[index].cur_id = 0xFF;
				}
      }
    }
	}
}

int GetTouchPosition( int aNo, int* aX, int* aY, int* aId )
{
  /* check for valid touch number */
  if (( aNo < 0 ) || ( aNo >= NUM_TP ))
    return 0;

  if ( aX )
    *aX = (int)( TP_Table[ aNo ].cur_x );
  if ( aY )
    *aY = (int)( TP_Table[ aNo ].cur_y );
  if ( aId )
    *aId = (int)( TP_Table[ aNo ].cur_id );

  if (( TP_Table[ aNo ].cur_x > 0 ) && ( TP_Table[ aNo ].cur_y > 0 )
    && ( TP_Table[ aNo ].cur_id != 0xFF ))
    return 1;
  else
    return 0;
}



#endif
