/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2013 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: RGA_Port.h
*    Porting Layer Interface for RGA.
*
* - $Version: $ (=RGA_VERSION)
* - $ModuleRevision: {Masters}\RGA\driver\RZ\31_NoWindowNoWorkB_RStyled $
* - $Rev: $
* - $Date: 2017-08-21T13:46:41+09:00 $
************************************************************************/

#ifndef  RGA_PORT_H
#define  RGA_PORT_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "RGA_API_typedef.h"
#include  "RGA_Port_typedef.h"
#include  "clib_drivers_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Macro definitions
******************************************************************************/
/* in RGA_Port_typedef.h */

/******************************************************************************
Typedef definitions
******************************************************************************/
/* in RGA_Port_typedef.h */

/******************************************************************************
Variable Externs
******************************************************************************/
/* in RGA_Port_typedef.h */

/******************************************************************************
Functions Prototypes
******************************************************************************/

/***********************************************************************
* Function: drawCLUTImage
************************************************************************/
errnum_t  drawCLUTImage( uint8_t *SrcAddress, const graphics_image_t* image, frame_buffer_t* frame );


/***********************************************************************
* Function: R_RGA_CalcWorkBufferSize
*    Calculate size of hardware work buffer
*
* Arguments:
*    max_height_of_frame_buffer - int
*
* Return Value:
*    size of hardware work buffer
************************************************************************/
#ifdef  RGAH_VERSION

#define  R_RGA_CalcWorkBufferSize( max_height_of_frame_buffer ) \
	( (size_t)( RGA_WORK_BUFFER_MAX_DISPLAY_LIST + \
		RGA_WORK_BUFFER_STRIDE * R_CEIL_8U( max_height_of_frame_buffer ) * 4 * 2 ) )
R_STATIC_ASSERT_GLOBAL( RGA_WORK_BUFFER_HEIGHT_ALIGNMENT == 8, "" );  /* check of R_CEIL_8U */

#endif


/***********************************************************************
* Function: R_RGA_CalcWorkBufferSize2
*    Calculate size of hardware work buffer
*
* Arguments:
*    max_width_of_frame_buffer  - int
*    max_height_of_frame_buffer - int
*
* Return Value:
*    size of hardware work buffer
************************************************************************/
#ifdef  RGAH_VERSION

#define  R_RGA_CalcWorkBufferSize2( max_width_of_frame_buffer, max_height_of_frame_buffer ) \
	R_RGA_CalcWorkBufferSize( max_height_of_frame_buffer )
#else

#define  R_RGA_CalcWorkBufferSize2( max_width_of_frame_buffer, max_height_of_frame_buffer ) \
	( (size_t)( (max_width_of_frame_buffer) * RGA_WORK_BUFFER_DMAC_LM_WIDTH_BYTE + \
		(max_height_of_frame_buffer) * RGA_WORK_BUFFER_DMAC_LM_HEIGHT_BYTE ) )
#endif


/***********************************************************************
* Function: R_RGA_CalcWorkBufferB_Size
*    Calculate size of hardware work buffer B
*
* Arguments:
*    MaxWidthOfJPEG - int
*    MaxHeightOfJPEG - int
*    MaxBytePerPixelOfFrameBuffer - int
*
* Return Value:
*    size of hardware work buffer B
************************************************************************/
#define  R_RGA_CalcWorkBufferB_Size( MaxWidthOfJPEG, MaxHeightOfJPEG, MaxBytePerPixelOfFrameBuffer ) \
	( R_CEIL_16U( MaxWidthOfJPEG ) * R_CEIL_16U( MaxHeightOfJPEG ) * (MaxBytePerPixelOfFrameBuffer) )
R_STATIC_ASSERT_GLOBAL( RGA_JPEG_MAX_WIDTH_ALIGNMENT  == 16, "" );  /* check of R_CEIL_16U */
R_STATIC_ASSERT_GLOBAL( RGA_JPEG_MAX_HEIGHT_ALIGNMENT == 16, "" );  /* check of R_CEIL_16U */


/***********************************************************************
* Function: R_GRAPHICS_OnInterrupting
************************************************************************/
errnum_t  R_GRAPHICS_OnInterrupting(void);


/***********************************************************************
* Function: R_GRAPHICS_STATIC_WaitForJCU
************************************************************************/
errnum_t  R_GRAPHICS_STATIC_WaitForJCU( bit_flags32_t  in_WaigingFlags,
                                        bit_flags32_t* const  out_GotFlags,  uint32_t const  in_Timeout_msec );


#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

/* Inline Functions */
#include  "RGA_Port_inline.h"

#endif  /* RGA_PORT_H */
