/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2013 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: RGA_Port_inline.h
*    This file is included from "RGA_Port.h".
*
* - $Version: $ (=RGA_VERSION)
* - $ModuleRevision: {Masters}\RGA\driver\RZ\31_NoWindowNoWorkB_RStyled $
* - $Rev: $
* - $Date: 2017-08-21T13:46:41+09:00 $
******************************************************************************/

#ifndef  RGA_PORT_INLINE_H
#define  RGA_PORT_INLINE_H
#ifndef  NOT_DEFINE_INLINE_FUNCTION

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "clib_drivers.h"
#include  "frame_buffer.h"
#include  "RGA_Port_typedef.h"

#ifdef  __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Macro definitions
******************************************************************************/
/* in "RGA_Port_typedef.h" */

/******************************************************************************
Typedef definitions
******************************************************************************/
/* in "RGA_Port_typedef.h" */

/******************************************************************************
Variable Externs
******************************************************************************/
/* in "RGA_Port_typedef.h" */

/******************************************************************************
Functions Prototypes
******************************************************************************/

/***********************************************************************
* Function: SwapEndian2Byte
*    SwapEndian to 2Byte
*
* Arguments:
*    in_out_2byteVariable - Address to 2 byte
*
* Return Value:
*    None
************************************************************************/
INLINE void  SwapEndian2Byte( void* in_out_2byteVariable )
{
    byte_t   buffer[2];
    byte_t*  data = (byte_t*) in_out_2byteVariable;

    buffer[0] = data[0];
    buffer[1] = data[1];

    data[0] = buffer[1];
    data[1] = buffer[0];
}


/***********************************************************************
* Function: SwapEndian4Byte
*    SwapEndian to 4Byte
*
* Arguments:
*    in_out_2byteVariable - Address to 4 byte
*
* Return Value:
*    None
************************************************************************/
INLINE void  SwapEndian4Byte( void* in_out_4byteVariable )
{
    byte_t   buffer[4];
    byte_t*  data = (byte_t*) in_out_4byteVariable;

    buffer[0] = data[0];
    buffer[1] = data[1];
    buffer[2] = data[2];
    buffer[3] = data[3];

    data[0] = buffer[3];
    data[1] = buffer[2];
    data[2] = buffer[1];
    data[3] = buffer[0];
}


/***********************************************************************
* Function: uint32_not_aligned_t_set
*    Sets the value to the address not aligned 4byte
*
* Arguments:
*    out_Variable - <uint32_not_aligned_t>
*    Value - Set value
*
* Return Value:
*    None
************************************************************************/
INLINE void  uint32_not_aligned_t_set( uint32_not_aligned_t* out_Variable,  uint32_t in_Value )
{
#if BYTE_ENDIAN == BYTE_LITTLE_ENDIAN
    out_Variable->byte[0] = (byte_t)( ( in_Value & 0x000000FF ) );
    out_Variable->byte[1] = (byte_t)( ( in_Value & 0x0000FF00 ) >>  8 );
    out_Variable->byte[2] = (byte_t)( ( in_Value & 0x00FF0000 ) >> 16 );
    out_Variable->byte[3] = (byte_t)( ( in_Value & 0xFF000000 ) >> 24 );
#else
    out_Variable->byte[0] = (byte_t)( ( in_Value & 0xFF000000 ) >> 24 );
    out_Variable->byte[1] = (byte_t)( ( in_Value & 0x00FF0000 ) >> 16 );
    out_Variable->byte[2] = (byte_t)( ( in_Value & 0x0000FF00 ) >>  8 );
    out_Variable->byte[3] = (byte_t)( ( in_Value & 0x000000FF ) );
#endif
}


/***********************************************************************
* Class: byte_per_pixel_t
************************************************************************/


/***********************************************************************
* Function: R_RGA_BitPerPixelType_To_BytePerPixelType
*    R_RGA_BitPerPixelType_To_BytePerPixelType
*
* Arguments:
*    in_BitPerPixel - Input
*
* Return Value:
*    byte_per_pixel_t
************************************************************************/
INLINE byte_per_pixel_t  R_RGA_BitPerPixelType_To_BytePerPixelType( int_fast32_t  in_BitPerPixel )
{
    if ( R_Mod_8s( in_BitPerPixel ) == 0 )
    {
        return  (byte_per_pixel_t)( (uint_fast32_t) in_BitPerPixel / 8 );
    }
    else
    {
        return  in_BitPerPixel << R_BYTE_PER_PIXEL_SHIFT;
    }
}


/***********************************************************************
* Function: R_RGA_BytePerPixelType_To_BitPerPixelType
*    R_RGA_BytePerPixelType_To_BitPerPixelType
*
* Arguments:
*    in_BytePerPixel - byte_per_pixel_t
*
* Return Value:
*    BitPerPixel
************************************************************************/
INLINE int_fast32_t  R_RGA_BytePerPixelType_To_BitPerPixelType( byte_per_pixel_t  in_BytePerPixel )
{
    return  (int_fast32_t)(
                ( ( (bit_flags_fast32_t) in_BytePerPixel & R_BYTE_PER_PIXEL_MASK ) * 8 ) |
                (bit_flags_fast32_t)( in_BytePerPixel >> R_BYTE_PER_PIXEL_SHIFT ) );
}


/***********************************************************************
* Function: R_BYTE_PER_PIXEL_IsInteger
*    R_BYTE_PER_PIXEL_IsInteger
*
* Arguments:
*    in_BytePerPixel - byte_per_pixel_t
*
* Return Value:
*    Whether "BytePerPixel" is integer type.
************************************************************************/
INLINE bool_t  R_BYTE_PER_PIXEL_IsInteger( byte_per_pixel_t in_BytePerPixel )
{
    return  ( (bit_flags_fast32_t) in_BytePerPixel & R_BYTE_PER_PIXEL_MASK ) != 0;
}


#ifdef  __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif  /* NOT_DEFINE_INLINE_FUNCTION */
#endif  /* RGA_PORT_INLINE_H */
