/* Support files for GNU libc.  Files in the system namespace go here.
   Files in the C namespace (ie those that do not start with an
   underscore) go in .c.  */

#include <stdio.h>
#include <errno.h>
#include "r_typedefs.h"
#include "r_sysIf.h"
#include "sio_char.h"

/* Forward prototypes.  */
#ifndef _PARAMS
#define _PARAMS(paramlist)		paramlist
#endif

caddr_t _sbrk		_PARAMS ((int));

int     _write 		_PARAMS ((int, char *, int));

int     _read		_PARAMS ((int, char *, int));

/* The following code gets the Newlib heap details from the linker
 * for use in the sbrk() allocation function.
 */
#define NL_HEAP_START          (&_nl_heap_start)
#define NL_HEAP_END            (&_nl_heap_end)

extern uint8_t _nl_heap_start;
extern uint8_t _nl_heap_end;

static char *pnl_heap_pointer =  (char*)(NL_HEAP_START);


/* _sbrk - this is used as a basic memory allocator over and above to the OS level malloc.
 * This should only be used by calls from the Newlib. A section has been defined in
 * the linker to provide dedicated memory for this heap - NL_HEAP.
 */
caddr_t
_sbrk (int incr)
{
	  char * prev_heap_end;
      int iMask = sysLock(NULL);

	  prev_heap_end = pnl_heap_pointer;

	  if (pnl_heap_pointer + incr > (char*)NL_HEAP_END)
	  {
		  errno = ENOMEM;
	      sysUnlock(NULL, iMask);
	      return (caddr_t) -1;
	  }

	  pnl_heap_pointer += incr;

      sysUnlock(NULL, iMask);
	  return (caddr_t) prev_heap_end;
}

/******************************************************************************
* Function Name: _write
* Description  : GNU interface to low-level I/O write
* Arguments    : int file_no
*              : const char *buffer
*              : unsigned int n
* Return Value : none
******************************************************************************/
int _write(int file_no , char *buffer , int n)
{
    return SioWrite(file_no , buffer , (uint32_t) n);
}


/******************************************************************************
* Function Name: _read
* Description  : GNU interface to low-level I/O read
* Arguments    : int file_no
*              : const char *buffer
*              : unsigned int n
* Return Value : none
******************************************************************************/
int _read(int file_no , char *buffer , int n)
{
    return SioRead(file_no , buffer , (uint32_t) n);
}
