/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
*******************************************************************************/
/*******************************************************************************
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File Name     : resetprg.c
* Device(s)     : RZ/A1H RSK+RZA1H
* Tool-Chain    : GNUARM-RZv13.01-EABI
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : Sample Program - C library entry point
*               : Variants of this file must be created for each compiler
******************************************************************************/
/******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 18.06.2013 1.00
******************************************************************************/
/* Interchangeable compiler specific header */
#include <stdint.h>
#include "irq.h"

#include "r_typedefs.h"
#include "stb_init.h"
#include "sio_char.h"

#include "peripheral_early_init.h"
#include "cache.h"
#include "cp15_access.h"
#include "devdrv_intc.h"
#include "intc_handler.h"
#include "rsk_switches.h"
#include "riic_rskrza1h.h"
#include "devdrv_common.h"
#include "port_init.h"
#include "peripheral_init_basic.h"
/* These are related to the code which runs at startup */
/* Global variables are specified in the linker script (GCC-QSPI-BL.ld) */
extern uint32_t __fast_code_rom_start__;
extern uint32_t __fast_code_start__;
extern uint32_t __fast_code_end__;

extern uint32_t __vector_table_ram_start__[];
extern uint32_t __vector_table_rom_start__;

/* These are related to the rest of the code */
extern uint32_t __data_rom_start__;
extern uint32_t __data_start__;
extern uint32_t __data_end__;

extern uint32_t __bss_start__;
extern uint32_t __bss_end__;



extern uint32_t reset_handler;
extern uint32_t undefined_handler;
extern uint32_t FreeRTOS_SWI_Handler;
extern uint32_t prefetch_handler;
extern uint32_t abort_handler;
extern uint32_t reserved_handler;
extern uint32_t FreeRTOS_IRQ_Handler;
extern uint32_t fiq_handler;

static void copy_arm_code_section_to_ram(uint32_t* rom_start, uint32_t* ram_start, uint32_t* ram_end);
extern int main(void);

static uint32_t const LDR_PC_PC = 0xE59FF000U;

/* note: no global or initialized variables can be used since .data is not yet initialized */
void PowerON_Reset (void)
{
    uint32_t *ram_code_start,*ram_code_end;
	uint32_t *rom_code_start;
	uint32_t volatile *VectorTableEntry;
	uint32_t volatile *VectorTableEntry_ROM;
	uint8_t  *src, *dst, *end;

	/* Now start initializing the hardware */
	CPG_Init();
	CS0_PORTInit();
	// enable all module clocks

	/* ==== BSC setting ==== */
	/* Initialise rest of bsc later in startup sequence */
   R_BSC_Init((uint8_t)(BSC_AREA_CS0 | BSC_AREA_CS1));
	STB_Init();

	PORT_Init();

	R_BSC_Init((uint8_t)(BSC_AREA_CS2 | BSC_AREA_CS3));

    L1CacheInit();

    L2CacheInit();

	/* data and bss are copied into initsct.S */
	/* relocate code and vector table to ram */
	/* these are the destination (run-time) locations of the code */
	ram_code_start = &__fast_code_start__;
	ram_code_end = &__fast_code_end__;

    /* this is where in rom the section is located */
	rom_code_start = &__fast_code_rom_start__;

	/* warning this is copying in multiple of 4 bytes so not suitable for 16-Bit thumb */
	copy_arm_code_section_to_ram(rom_code_start, ram_code_start, ram_code_end);

	/* setup any of the other .data, .const or .bss initialized sections */
	src = (uint8_t*) &__data_rom_start__;
	dst = (uint8_t*) &__data_start__;
	end = (uint8_t*) &__data_end__;

	/* if not debugging directly from RAM */
	if(src != dst) {
		/* ROM has data at end of text; copy it. */
		while (dst < end) {
			*dst++ = *src++;
		}
	};

	dst = (uint8_t*) &__bss_start__;
	end = (uint8_t*) &__bss_end__;

	/* Zero bss */
	for (; dst< end; dst++)
		*dst = 0;

	/* now C environment is properly configured */

	/* copy vector table to ram */
	VectorTableEntry = (uint32_t volatile *) &__vector_table_ram_start__;
	VectorTableEntry_ROM = (uint32_t volatile *) &__vector_table_rom_start__;

	// do this only if not debugging from RAM
	if(VectorTableEntry != VectorTableEntry_ROM) {

		/* setup the primary vector table in RAM */
		*(VectorTableEntry++) = (LDR_PC_PC | 0x18);
		*(VectorTableEntry++) = (LDR_PC_PC | 0x18);
		*(VectorTableEntry++) = (LDR_PC_PC | 0x18);
		*(VectorTableEntry++) = (LDR_PC_PC | 0x18);
		*(VectorTableEntry++) = (LDR_PC_PC | 0x18);
		*(VectorTableEntry++) = (LDR_PC_PC | 0x18);
		*(VectorTableEntry++) = (LDR_PC_PC | 0x18);
		*(VectorTableEntry++) = (LDR_PC_PC | 0x18);

		/* setup the literal entries for the vector table in RAM */
		*(VectorTableEntry++) = (uint32_t) &reset_handler; 			/* this is in QSPI */
		*(VectorTableEntry++) = (uint32_t) &undefined_handler;  	/* self loop could be coded as 0x04U */
		*(VectorTableEntry++) = (uint32_t) &FreeRTOS_SWI_Handler;	/* self loop could be coded as 0x08U */
		*(VectorTableEntry++) = (uint32_t) &prefetch_handler; 		/* self loop could be coded as 0x0CU */
		*(VectorTableEntry++) = (uint32_t) &abort_handler;			/* self loop could be coded as 0x10U */
		*(VectorTableEntry++) = (uint32_t) &reserved_handler;		/* self loop could be coded as 0x14U */
		*(VectorTableEntry++) = (uint32_t) &FreeRTOS_IRQ_Handler;	/* IRQ and FIQ handler are located in RAM */
		*(VectorTableEntry++) = (uint32_t) &fiq_handler;

		L1_D_CacheWritebackAll();

		// set VBAR to the vector table in RAM
		VectorTableEntry = (uint32_t volatile *) &__vector_table_ram_start__;
		VbarSet((uint32_t) VectorTableEntry);

	};

    /* INTC setting */
    R_INTC_Init();

    __enable_irq();
    __enable_fiq();

	/* Initialise user switches */
	init_switches();

    /* ==== Initialize Terminal ==== */
    /* SCIF 2ch */
    /* P1 clock=66.67MHz CKS=0 SCBRR=17 Bit rate error=0.46% => Baud rate=115200bps */
    IoInitScif2();

	/* Initialise ri2c for rsk board */
	R_RIIC_rza1h_rsk_init();

    /* main */
    main();

	/* Stops program from running off */
    while(1)
    {
		__asm__("nop");
   	}
}


void copy_arm_code_section_to_ram(uint32_t* rom_start, uint32_t* ram_start, uint32_t* ram_end)
{

	/* Variables for program copy */
    uint32_t i;
    uint32_t region_size;

    // check if executing already from ram (when debugging)
    if(rom_start == ram_start) return;

    /* Calculate the length of the copied section */
    region_size     = (uint32_t)ram_end - (uint32_t)ram_start;

    /* Copy the next load module. */
    for(i = 0; i < (region_size/4); i++)
    {
        *ram_start++ = *rom_start++;
    }

}

